/*******************************************************************************
 * Copyright 2011 John Casey
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.commonjava.tensor.mem.data.store;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.enterprise.context.ApplicationScoped;

import org.apache.maven.graph.effective.EProjectGraph;
import org.apache.maven.graph.effective.EProjectNet;
import org.apache.maven.graph.effective.EProjectRelationships;
import org.apache.maven.graph.effective.ref.EProjectKey;
import org.commonjava.tensor.data.TensorDataException;
import org.commonjava.tensor.data.store.TensorDataStore;

@ApplicationScoped
public class MemoryTensorDataStore
    implements TensorDataStore
{

    private final Map<EProjectKey, EProjectRelationships> effectiveRelationships =
        new HashMap<EProjectKey, EProjectRelationships>();

    private final Map<EProjectKey, EProjectGraph> graphs = new HashMap<EProjectKey, EProjectGraph>();

    @Override
    public EProjectRelationships storeRelationshipSet( final EProjectRelationships relationships, final boolean force )
        throws TensorDataException
    {
        if ( force )
        {
            return effectiveRelationships.put( relationships.getKey(), relationships );
        }
        else if ( !effectiveRelationships.containsKey( relationships.getKey() ) )
        {
            return effectiveRelationships.put( relationships.getKey(), relationships );
        }

        return null;
    }

    @Override
    public EProjectNet storeGraph( final EProjectGraph graph, final boolean force )
        throws TensorDataException
    {
        if ( force )
        {
            return graphs.put( graph.getKey(), graph );
        }
        else if ( !graphs.containsKey( graph.getKey() ) )
        {
            return graphs.put( graph.getKey(), graph );
        }

        return null;
    }

    @Override
    public boolean containsRelationshipSet( final EProjectKey key )
    {
        return effectiveRelationships.containsKey( key );
    }

    @Override
    public boolean containsGraph( final EProjectKey key )
    {
        return graphs.containsKey( key );
    }

    @Override
    public EProjectGraph getGraph( final EProjectKey key )
    {
        return graphs.get( key );
    }

    @Override
    public EProjectRelationships getRelationshipSet( final EProjectKey key )
    {
        return effectiveRelationships.get( key );
    }

    @Override
    public Set<EProjectGraph> getAllGraphs()
    {
        return new HashSet<EProjectGraph>( graphs.values() );
    }

    @Override
    public Set<EProjectRelationships> getAllRelationshipSets()
    {
        return new HashSet<EProjectRelationships>( effectiveRelationships.values() );
    }

    @Override
    public Set<EProjectKey> getGraphKeys()
    {
        return graphs.keySet();
    }

    @Override
    public Set<EProjectKey> getRelationshipSetKeys()
    {
        return effectiveRelationships.keySet();
    }

}
